/** 
 * Copyright (c) 1998, 2021, Oracle and/or its affiliates. All rights reserved.
 * 
 */

package hwb1;

import javacard.framework.*;
/**
 * Applet class
 * 
 * @author <user>
 */
public class MyApplet extends Applet {
    final static byte PIN_TRY_LIMIT =(byte)0x03;
    final static byte MAX_PIN_SIZE =(byte)0x08;
    private byte[] authorName = new byte[] { 'M', 'a', 'n', 'u', 'e', 'l' };
    private byte[] storage = new byte[] {};
    OwnerPIN pin;

    /**
     * Installs this applet.
     * 
     * @param bArray  the array containing installation parameters
     * @param bOffset the starting offset in bArray
     * @param bLength the length in bytes of the parameter data in bArray
     */
    public static void install(byte[] bArray, short bOffset, byte bLength) {
        new MyApplet(bArray, bOffset, bLength);
    }

    /**
     * Only this class's install method should create the applet object.
     *
     * @param bArray  the array containing installation parameters
     * @param bOffset the starting offset in bArray
     * @param bLength the length in bytes of the parameter data in bArray
     */
    protected MyApplet(byte[] bArray, short bOffset, byte bLength) {
        pin = new OwnerPIN(PIN_TRY_LIMIT, MAX_PIN_SIZE);
        pin.update(bArray, bOffset, bLength);
        register();
    }

    public boolean select() {
        if (pin.getTriesRemaining() == 0) {
            return false;
        } else {
            return super.select();
        }
    }

    /**
     * Processes an incoming APDU.
     * 
     * @see APDU
     * @param apdu the incoming APDU
     */
    public void process(APDU apdu) {
        short length;
        byte[] buffer = apdu.getBuffer();
        
        if (selectingApplet()) {
            ISOException.throwIt(ISO7816.SW_NO_ERROR);
        } else {
            if (buffer[ISO7816.OFFSET_CLA] != (byte)0x80) {
                ISOException.throwIt(ISO7816.SW_CLA_NOT_SUPPORTED);
            } else {
                byte instruction = buffer[ISO7816.OFFSET_INS];

                switch (instruction) {
                    case 0x00:
                    case 0x04:
                        byte[] response;
                        length = apdu.setOutgoing();

                        if (instruction == 0x00) {
                            response = authorName;
                            length = (short)response.length;
                        } else {
                            response = storage;

                            if (length > response.length) {
                                ISOException.throwIt((short)(ISO7816.SW_CORRECT_LENGTH_00 | response.length));
                            }
                        }

                        apdu.setOutgoingLength(length);
                        apdu.sendBytesLong(response, (short) 0, length);
                        break;
                    case 0x02:
                        length = apdu.setIncomingAndReceive();
                        storage = new byte[length];

                        if (length > 20) {
                            ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
                        } else {
                            Util.arrayCopy(buffer, ISO7816.OFFSET_CDATA, storage, (short) 0, length);
                        }
                        break;
                    case 0x20:
                        length = apdu.setIncomingAndReceive();
                        pin.check(buffer, (short) ISO7816.OFFSET_CDATA, (byte)length); 
                    default:
                        ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
                        break;
                }
                
                ISOException.throwIt(ISO7816.SW_NO_ERROR);
            }
        }
    }
}
